/*  www.sgm.siedlce.pl, program do wykorzystania tylko na wlasny uzytek

- 2-kanałowy RC Switch (sygnal aktywny HIGH)
  na wyjsciach D11, D12
- sygnal wejsciowy z odbiornika RC na wejsciu D2
  z kanalu aparatury z przelacznikiem 3-polozeniowym
- pozycja spoczynkowa - środkowa (1500uS), pozycje robocze
  minimum lub maksimum
- przy braku sygnalu uklad dziala jak dla pozycji srodkowej  
- funkcja uzbrajania - jesli po wlacznieniu przelacznik
  nadajnika nie znajuduje sie w pozycji srodkowej, 
  uklad dziala jak dla pozycji srodkowej
- dioda LED na plytce ardu (D13) sygnalizuje stan wejscia
  pomiarowego (przelacznika SW w aparaturze):
  miganie szybkie  - SW=Mid  (>1300uS i <1700uS)
  miganie powolne  - SW=Low  (<1300uS)
  swiecenie ciagle - SW=High (>1700uS) 
  zgaszona         - brak sygnalu RC 
- zworka na wejściu A1 ustala tryb pracy (mode):  
   Mode 0 w stanie Mid oba wyjscia wlaczone,
          przesuniecie SW wylacza jedno wyjscie;
   Mode 1 w stanie Mid oba wyjscia wylaczone,
          przesuniecie SW wlacza jedeno wyjscie;

- pozostale uwagi:          

   pomiar kanalu RC na wejsciu (D2)
   w oparciu o przerwanie; obsługa przerwania --> procedury:
   void pulse_rising()
   void pulse_falling()

   dozwolony zakres dlugosci impulsu: 900-2100uS

   opcjonalnie wydruk zmierzonych wartosci na port szeregowy

   pomiar czasu przy uzyciu biblioteki standardowej funkcji micros()

   usrednianie zmierzonego sygnału (4 ostatnie probki)

*/




//deklaracje zmiennych

volatile unsigned long pulse1_start = 0;   //zmienne dla micros()
unsigned long pulse1_micros = 0;           //zmienna na dlugosc impulsu
                                 //kanalowego PPM wg micros()

int pulse_uS;                    //zmienne pomocnicze
int servoTmp;
int buf1;                        //zmienne bufora impulsow
int buf2;
int buf3;

byte rc_input;               //cz jest impuls RC na wejsciu D2

byte rc_mode;                //mode switch'a w zaleznosci od zworki
                             //0 w stanie Mid oba przekazniki wlaczone,
                             //  przesuniecie SW wylacza jeden przekaznik
                             //1 w stanie Mid oba przekazniki wylaczone,
                             //  przesuniecie SW wlacza jeden przekaznik                             //1 

byte SW_start;               //do testu SW po starcie programu
                             //SW_start=2 gotowy w polozeniu Mid
byte SW_mode;                //zmienna do sterowania przekaznikami
byte SW_pos;                 //zmienna na polozenie przelacznika RC
const int SW_low = 1300;         //stala - SW w polozeniu dolnym
const int SW_high = 1700;        //stala - SW w polozeniu gornym
const int LED = 13;              // LED na Pin 13
byte LED_status = 0;             //status diody LED
const int Relout1 = 11;            // out1 na Pin 11
const int Relout2 = 12;            // out1 na Pin 11

int servoP;
const int servoFS = 1500;        //wartosc domyslna impulsu RC

const int tGap = 250;            //opcjonalnie - co ile wydruk danych (msec)
unsigned long t_start;           //zmienne do odliczania czasu przy wydruku
unsigned long t_count; 

const int tGapLL = 500;          //stale i zmienne do obslugi migania diody
const int tGapLH = 200;
unsigned long t_startL;
unsigned long t_countL;  




//inicjalizacja programu
void setup() { 

  // inicjalizacja zmiennych pomiaru

  rc_input = 0;
  rc_mode = 0;
  SW_pos = 0;
  SW_start =0;
  SW_mode = 1;
  servoP = servoFS;
  buf1 = servoFS;
  buf2 = buf1;
  buf3 = buf2;
  
  //odczyt stanu zworki
  int TstZW = analogRead(1); //odczyt A1
  if (TstZW <500){
   rc_mode = 0;   //jest 
  } else {
   rc_mode = 1;   //brak zworki
  }
  
  // inicjalizacja wyjsc przekaznikow
  
  if (rc_mode == 0) {
   digitalWrite(Relout1, HIGH);
   digitalWrite(Relout2, HIGH);  
  }
   if (rc_mode == 1) {
   digitalWrite(Relout1, LOW);
   digitalWrite(Relout2, LOW);  
  }
    
  
  //inicjalizacja portu szeregowego (opcjonalny wydruk danych)
  //Serial.begin(115200); 
  
  //inicjalizacja przerwania 0 (D2, Arduino UNO)
  attachInterrupt(0, pulse_rising, RISING);
  t_start = millis();  
}

//glowna petla programu
void loop() {


// sekwencja startowa odczytu polozenia SW
  if (SW_start == 0) {
   delay(1000);
   SW_start = 1; 
  }
  
  
  // opcjonalnie do testow obsulga drukowania
  // pomiaru na port szeregowy co czas 'tGap'   
  /*
  t_count = millis();
  
  //jesli uplynelo tGap to drukuj
  if ((t_count - t_start) >= tGap) 
  {
    t_start = t_count;
    Serial.print(servoP);
    Serial.print(",  sw");
    Serial.print(SW_pos);
    Serial.print(",  st");
    Serial.print(SW_start);
    Serial.print(",  M");
    Serial.println(SW_mode);
  }
  
  */
 
  if (rc_input == 1){  
   //obsluga diody sygnalizujacej D13
   switch (SW_pos) {  
    
    case 0:          // polozenie LOW, dioda miga powoli
     t_countL = millis();
     if ((t_countL - t_startL) >= tGapLL) {
      t_startL = t_countL; 
      if (LED_status == 0) {
       digitalWrite(LED, HIGH);
       LED_status = 1; 
      } else {
       digitalWrite(LED, LOW);
       LED_status = 0;
       rc_input = 0;  //zerowanie do testu wejscia po zgaszeniu LED
      } 
     } //if ((t_countL      
    break; // case 0
   
    case 1:          // polozenie MID, dioda miga szybko
     t_countL = millis();
     if ((t_countL - t_startL) >= tGapLH) {
      t_startL = t_countL; 
      if (LED_status == 0) {
       digitalWrite(LED, HIGH);
       LED_status = 1; 
      } else {
       digitalWrite(LED, LOW);
       LED_status = 0;
       rc_input = 0;  //zerowanie do testu wejscia po zgaszeniu LED
      } 
     } //if ((t_countL 
    break; // case 1
    
    case 2:          // polozenie HIGH, dioda swieci ciagle
     digitalWrite(LED, HIGH);
     LED_status = 1;
     t_countL = millis();
     if ((t_countL - t_startL) >= tGapLL) {
      t_startL = t_countL;
      rc_input = 0;  //zerowanie do testu wejscia po tGapLL
     }
    break; // case 2    
   
   } //switch (SW_pos)
   //koniec obslugi diody
  } else {
   digitalWrite(LED, LOW);
  }  

   
   if ((SW_start == 1) && (SW_pos == 1)) {
     SW_start = 2;  //SW gotowy w polozeniu Mid
   }
 
   if (SW_start == 2) {
     SW_mode = SW_pos;
   } 
   
   
   switch (SW_mode) { 

    case 0:          // polozenie LOW,
     if (rc_mode == 0) {
      digitalWrite(Relout1, LOW);
      digitalWrite(Relout2, HIGH);  
     }
     if (rc_mode == 1) {
      digitalWrite(Relout1, HIGH);
      digitalWrite(Relout2, LOW);  
     } 
    
    break; // case 0  
    
    case 1:          // polozenie MID,
     if (rc_mode == 0) {
      digitalWrite(Relout1, HIGH);
      digitalWrite(Relout2, HIGH);  
     }
     if (rc_mode == 1) {
      digitalWrite(Relout1, LOW);
      digitalWrite(Relout2, LOW);  
     } 
    break; // case 2 
     
    case 2:          // polozenie HIGH, 
     if (rc_mode == 0) {
      digitalWrite(Relout1, HIGH);
      digitalWrite(Relout2, LOW);  
     }
     if (rc_mode == 1) {
      digitalWrite(Relout1, LOW);
      digitalWrite(Relout2, HIGH);  
     }
    break; // case 2  
     
   } //switch (SW_mode)
 
  
} // koniec glownej petli



// obsluga przerwania (rising), poczatek impulsu
void pulse_rising() {
  pulse1_start = micros();
  attachInterrupt(0, pulse_falling, FALLING);
  rc_input = 1;  
}


// obsluga przerwania (falling), koniec impulsu 
void pulse_falling() {
  //dlugosc impulsu wg micros()
  pulse1_micros = micros()-pulse1_start;
  pulse_uS = pulse1_micros;
  //sprawdzenie czy dlugosc impulsu z dozwolonego zakresu
  if ((pulse_uS > 900) && (pulse_uS < 2100)) {
     servoTmp = pulse_uS + buf3 + buf2 + buf1; 
     servoP = servoTmp / 4;
     buf3 = buf2;
     buf2 = buf1;
     buf1 = servoP;
     
     if ((servoP > SW_low) && (servoP < SW_high)) {
      SW_pos = 1;  
     }
     if (servoP <= SW_low)  {
      SW_pos = 0;  
     }
     if (servoP >= SW_high) {
      SW_pos = 2;  
     } 
     //myservo.writeMicroseconds(servoP);
  }//if ((pulse_uS > 900
 
  attachInterrupt(0, pulse_rising, RISING);        
}





